//---------------------------------------------------------------------------
// MODULO DE PROCESAMIENTO DE IMAGENES
// PRACTICA FotoPain, versin 2009
// Procesamiento Audiovisual, 3 II, ITIS
// Universidad de Murcia, GinesGM
//---------------------------------------------------------------------------

#ifndef Unit2H
#define Unit2H
//---------------------------------------------------------------------------
#include <ipl.h>
#include <cv.h>
#include <highgui.h>
//---------------------------------------------------------------------------

#include <vcl.h>


///////////////////////////////////////////////////////////////////
/////////  DEFINICIONES PUBLICAS                     //////////////
///////////////////////////////////////////////////////////////////

#define MAX_VENTANAS 50

#define min(a, b)      (((a) < (b)) ? (a) : (b))
#define max(a, b)      (((a) > (b)) ? (a) : (b))


///////////////////////////////////////////////////////////////////
/////////  TIPOS DE DATOS PROPIOS                    //////////////
///////////////////////////////////////////////////////////////////

// STRUCT ventana
//    Representa la informacin asociada a una imagen que est siendo
//    editada actualmente en la aplicacin

struct ventana {
  bool usada;          // Indica si se est usando actualmente esta posicin
  bool modificada;     // Imagen modificada despus de abrirla o crearla
  char *nombre;        // Nombre del fichero que contiene la imagen
  HWND handle;         // Manejador de Windows de la ventana de HighGUI
  CvRect roi;          // Regin de inters seleccionada (-1 si no hay ninguna)
  IplImage *img;       // La imagen en s
};

// ENUM tipo_herramienta
//    Enumerado con los distintos tipos posibles de herramientas
//    que se pueden usar. Aadir nuevas aqu

enum tipo_herramienta {puntos, lineas, seleccionar};


///////////////////////////////////////////////////////////////////
/////////  VARIABLES PBLICAS                        //////////////
///////////////////////////////////////////////////////////////////

extern ventana foto[MAX_VENTANAS];
// El array foto guarda la informacin de todas las imgenes abiertas
// actualmente o que pueden ser abiertas, hasta un mximo de MAX_VENTANAS

extern tipo_herramienta herr_actual;
// Herramienta que se ha seleccionado actualmente

extern int radio_pincel;
// Radio actual para pintar puntos y lneas

extern CvScalar color_pincel;
// Color actual del pincel

extern int difum_pincel;
// Difuminado actual del pincel

extern bool preguntar_guardar;
// Al cerrar una imagen modificada, preguntar si se guarda o no


///////////////////////////////////////////////////////////////////
/////////  FUNCIONES DE MANEJO DE VENTANAS           //////////////
///////////////////////////////////////////////////////////////////

void inic_fotos (void);
// Inicializa el array foto, de imgenes abiertas actualmente

void fin_fotos (void);
// Libera las imgenes del array foto

int primera_libre (void);
// Busca la primera entrada libre del array foto

void crear_nueva (int nfoto, int ancho, int alto, CvScalar color);
// Crea una nueva entrada, en la posicin nfoto, con el ancho y el alto dados,
// de profundidad IPL_DEPTH_8U y 3 canales. color indica el color de fondo

void crear_nueva (int nfoto, IplImage *img);
// Crea una nueva entrada, en la posicin nfoto, usando la imagen dada en img

void crear_nueva (int nfoto, char *nombre);
// Crea una nueva entrada, en la posicin nfoto, leyndola del archivo nombre.
// Si no se puede leer, no hace nada

void mostrar (int nfoto);
// Muestra la imagen de la entrada nfoto en la ventana correspondiente

void guardar_foto (int nfoto, char *nombre= NULL);
// Guarda en disco la imagen de nfoto, con el nombre dado. Si el nombre
// es NULL, usa el nombre de la imagen en el array foto. Si cerrar es false,
// se cierra la ventana con la imagen (modo "guardar y cerrar")

void cerrar_foto (int nfoto, bool destruir_ventana= true);
// Elimina la entrada en la posicin nfoto. destruir_ventana indica si debe
// destruirse la ventana de HighGUI (ya que si lo ha hecho el usuario, no
// debe volver a hacerse)

int nombre_a_numero (const char *nombre);
// Dado un nombre de ventana, busca su nmero en el array foto.
// Si no lo encuentra devuelve -1

int foto_activa (void);
// Devuelve el nmero de la ventana de imagen activa (la que est en primer
// lugar). Si no hay ninguna devuelve -1


///////////////////////////////////////////////////////////////////
/////////  FUNCIONES DE PROCESAMIENTO DE IMAGENES    //////////////
///////////////////////////////////////////////////////////////////

void invertir (int nfoto);
// Invierte la imagen contenida en foto_activa() y la almacena en nfoto

void rotar_90 (int nfoto);
// Rota 90 la imagen contenida en foto_activa() y la almacena en nfoto

void rotar_m90 (int nfoto);
// Rota -90 la imagen contenida en foto_activa() y la almacena en nfoto

void ver_brillo_contraste (int nfoto, int suma, double prod,
                           bool guardar= false);
// Modifica la imagen en nfoto, tomando para cada pxel A(x,y) el valor:
// A(x,y)*prod + suma. Si guardar==true se guarda la imagen, y en otro caso
// slo se visualiza el resultado, pero sin guardar

void ver_suavizado (int nfoto, int ntipo, int tamx, int tamy,
                    bool guardar= false);
// Modifica la imagen en nfoto, aplicando un suavizado de tamao dado. ntipo
// puede ser CV_GAUSSIAN o CV_BLUR. Si guardar==true se guarda la imagen, y
// en otro caso slo se visualiza el resultado, pero sin guardar

void media_ponderada (int nf1, int nf2, int nueva, double peso);
// Calcula la media ponderada entre la imagen nf1 y la nf2, segn el peso dado,
// y almacena el resultado en la posicin nueva. Se aplica la frmula:
// nueva(x,y):= peso*nf1(x,y)+(1-peso)*nf2(x,y). La imagen nf2 se reescala al
// tamao de nf1


#endif
