//---------------------------------------------------------------------------
// MODULO DE PROCESAMIENTO DE VIDEO
// PRACTICA FotoPain, versin 2009
// Procesamiento Audiovisual, 3 II, ITIS
// Universidad de Murcia, GinesGM
//---------------------------------------------------------------------------

#pragma hdrstop

#include "Unit9.h"
#include <math.h>
//---------------------------------------------------------------------------

#pragma package(smart_init)


///////////////////////////////////////////////////////////////////
/////////  VARIABLES GLOBALES PRIVADAS               //////////////
///////////////////////////////////////////////////////////////////

CvCapture *camara= NULL;
// Capturador de cmara usado actualmente

IplImage *img_media= NULL;
// Imagen media acumulada de la cmara

IplImage *frame_float= NULL;
// Suma de los frames acumulados en la cmara

int frames_img_media;
// Nmero de frames que se han acumulado en la media frame_float


///////////////////////////////////////////////////////////////////
/////////  FUNCIONES DE PROCESAMIENTO DE VIDEO       //////////////
///////////////////////////////////////////////////////////////////

void rotar_video (int nfoto, char *nombre, int modo, int nframes)
{
  assert(nfoto>=0 && nfoto<MAX_VENTANAS && foto[nfoto].usada);
  assert(nombre);
  assert(nframes>0);
  CvSize sres;
  double W= foto[nfoto].img->width, H= foto[nfoto].img->height;
  double rW2, rH2;
  double dx, dy, alfa;
  if (modo==0) {
    sres.width= W;
    sres.height= H;
  }
  else if (modo==1)
    sres.width= sres.height= sqrt(W*W + H*H);
  else
    sres.width= sres.height= min(W, H)/sqrt(2.0);
  rW2= sres.width/2.0;
  rH2= sres.height/2.0;
  IplImage* res= cvCreateImage(sres, foto[nfoto].img->depth, foto[nfoto].img->nChannels);
  CvVideoWriter* aw= cvCreateVideoWriter(nombre, -1, 25.0, sres);
  if (aw) {
    iplRotate(foto[nfoto].img, res, 0, 0, 0, IPL_INTER_CUBIC);
    cvNamedWindow("Imagen a vdeo - Rotacin", 1);
    for (int i= 0; i<nframes; i++) {
      alfa= M_PI*2.0*i/nframes;
      dx= rW2 - W/2*cos(alfa) - H/2*sin(alfa);
      dy= rH2 + W/2*sin(alfa) - H/2*cos(alfa);
      cvZero(res);
      iplRotate(foto[nfoto].img, res, 360.0*i/nframes, dx, dy, IPL_INTER_CUBIC);
      cvShowImage("Imagen a vdeo - Rotacin", res);
      cvWaitKey(1);
      cvWriteFrame(aw, res);
    }
    cvReleaseVideoWriter(&aw);
    cvDestroyWindow("Imagen a vdeo - Rotacin");
  }
  cvReleaseImage(&res);
}

//---------------------------------------------------------------------------

int inic_camara (int numero)
{
  assert(!camara);
  camara= cvCaptureFromCAM(numero);
  if (camara) {
    IplImage *img;
    cvNamedWindow("Imagen de cmara", 0);
    img= cvQueryFrame(camara);
    cvResizeWindow("Imagen de cmara", img->width, img->height);
    cvShowImage("Imagen de cmara", img);
    return 1000.0/30;
  }
  else
    return 0;
}

//---------------------------------------------------------------------------

void fin_camara (void)
{
  if (camara) {
    cvReleaseCapture(&camara);
    cvDestroyWindow("Imagen de cmara");
  }
  if (img_media) {
    cvReleaseImage(&img_media);
    cvDestroyWindow("Imagen media");
  }
}

//---------------------------------------------------------------------------

void acumular_media (bool primera)
{
  assert(camara);
  IplImage *frame= cvQueryFrame(camara);
  if (frame->origin) {
    cvFlip(frame);
    frame->origin= 0;
  }
  cvNamedWindow("Imagen de cmara", 0);
  cvShowImage("Imagen de cmara", frame);
  if (primera) {
    cvReleaseImage(&img_media);
    cvReleaseImage(&frame_float);
  }
  if (!img_media) {
    img_media= cvCreateImage(cvSize(frame->width, frame->height), IPL_DEPTH_32F, frame->nChannels);
    frame_float= cvCreateImage(cvSize(frame->width, frame->height), IPL_DEPTH_32F, frame->nChannels);
    cvConvert(frame, img_media);
    cvNamedWindow("Imagen media", 1);
    frames_img_media= 1;
    cvShowImage("Imagen media", frame);
  }
  else {
    frames_img_media++;
    cvScale(frame, frame_float);
    cvAdd(img_media, frame_float, img_media);
    cvConvertScale(img_media, frame, 1.0/frames_img_media);
    cvShowImage("Imagen media", frame);
  }
}

//---------------------------------------------------------------------------

void media_a_nueva (int nfoto)
{
  IplImage *res= cvCreateImage(cvSize(img_media->width, img_media->height),
                   IPL_DEPTH_8U, img_media->nChannels);
  cvConvertScale(img_media, res, 1.0/frames_img_media);
  crear_nueva (nfoto, res);
}

//---------------------------------------------------------------------------

void mostrar_camara (void)
{
  cvNamedWindow("Imagen de cmara", 0);
  IplImage *img= cvQueryFrame(camara);
  cvShowImage("Imagen de cmara", img);
}

//---------------------------------------------------------------------------

void dialogo_video (int tipo)
{
  cvSetCaptureProperty(camara, tipo, 0);
}

//---------------------------------------------------------------------------


